package hitachi.csmb.checker.gui;

import hitachi.csmb.checker.CsMbChecker;
import hitachi.csmb.checker.work.Iu;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.Timer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

public class DataPane extends JPanel implements ActionListener,ListSelectionListener{
	private DefaultTableModel dataModel;
	private JTable dataTable;
	private Vector<UpdateListener> updateListeners;
	
	private String colTitles[] = {
			"Id",				//0
			"Ou Add",			//1
			"Iu Add",			//2
			"On/Off",			//3
			"Mode",				//4
			"Fan",				//5
			"Tset",				//6
			"Louver",			//7
			"Central",			//8
			"On/Off Read",		//9
			"Mode Read",		//10
			"Fan Read",			//11
			"Tset Read",		//12
			"Louver Read",		//13
			"RCS Group",		//14
			"Tin",				//15
			"Tout",				//16
			"Tgas",				//17
			"Tliquid",			//18
			"Alarm",			//19
			"Comp Stop",		//20
			"EVI",				//21
			"Status",			//22
			"Defrost",			//23
			"Outdoor Temp",		//24
			"Power Group",		//25
			"Timer Disabled",	//26
			"Options"			//27
			
	};
	
	public void addUpdateListener(UpdateListener listener){
		updateListeners.add(listener);
	}
	
	public void removeUpdateListener(UpdateListener listener){
		updateListeners.remove(listener);
	}
	
	public void fireUpdateEvent(Iu iu){
		UpdateEvent event = new UpdateEvent(this);
		event.setIu(iu);
		for(int i=0;i<updateListeners.size();i++){
			updateListeners.get(i).updatePerformed(event);
		}
	}
	
	private void initialize(){
		updateListeners = new Vector<UpdateListener>();
		dataModel = new DefaultTableModel(){
			@Override
			public boolean isCellEditable(int arg0, int arg1) {
				// TODO Auto-generated method stub
				return false;
			}
		};
		dataTable = new JTable(dataModel);
		JScrollPane scrollPane = new JScrollPane(dataTable);
		
		dataModel.setColumnIdentifiers(colTitles);
		
		dataTable.getTableHeader().setResizingAllowed(true);
		//dataTable.getTableHeader().setPreferredSize(new Dimension(80,25));
		dataTable.getTableHeader().setReorderingAllowed(false);
		dataTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		dataTable.setRowHeight(25);
		dataTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		
		dataTable.setDefaultRenderer(Object.class,new ColoredCellRenderer());
		dataTable.setDefaultRenderer(String.class,new ColoredCellRenderer());
		
		dataTable.getSelectionModel().addListSelectionListener(this);
		
		scrollPane.getViewport().setBackground(Color.white);
		
		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
		
		int delay = 1000; //milliseconds
		new Timer(delay, this).start();
	}
	
	public void actionPerformed(ActionEvent evt) {
        updateData();
    }
	
	private void updateData(){
		Iu[] ius = CsMbChecker.worker.getData().getUnitArray();
		int iRow = -1;
		String selId = ""; 
		
		iRow = dataTable.getSelectedRow();
		if(iRow >=0) selId = dataTable.getValueAt(iRow, 0).toString();
		
		String[][] data = new String[ius.length][28];
		for(int i=0;i<ius.length;i++){
			Iu iu = ius[i];
			data[i][0] = ""+iu.getId();
			data[i][1] = ""+iu.getOuAdd();
			data[i][2] = ""+iu.getIuAdd();
			data[i][3] = ""+iu.getOnOffSet();
			data[i][4] = ""+iu.getModeSet();
			data[i][5] = ""+iu.getFanSet();
			data[i][6] = ""+iu.getTempSet();
			data[i][7] = ""+iu.getLouverSet();
			data[i][8] = ""+iu.getCentralSet();
			data[i][9] = ""+iu.getOnOffRead();
			data[i][10] = ""+iu.getModeRead();
			data[i][11] = ""+iu.getFanRead();
			data[i][12] = ""+iu.getTempRead();
			data[i][13] = ""+iu.getLouverRead();
			data[i][14] = ""+iu.getRcsGroup();
			data[i][15] = ""+iu.getTin();
			data[i][16] = ""+iu.getTout();
			data[i][17] = ""+iu.getTgas();
			data[i][18] = ""+iu.getTliquid();
			data[i][19] = ""+iu.getAlarm();
			data[i][20] = ""+iu.getOnOffSet();
			data[i][21] = ""+iu.getEv();
			data[i][22] = ""+iu.getStatus();
			data[i][23] = ""+iu.getDefrost();
			data[i][24] = ""+iu.getTOutdoor();
			data[i][25] = ""+iu.getPowerGroup();
			data[i][26] = ""+iu.getTimerDisabled();
			data[i][27] = ""+iu.getOptions();
		}
		dataModel.setDataVector(data, colTitles);
		
		for(int i=0;i<dataTable.getRowCount();i++){
			if(dataTable.getValueAt(i, 0).toString().equals(selId)){
				dataTable.getSelectionModel().setSelectionInterval(i, i);
				fireUpdateEvent(ius[i]);
				return;
			}
		}
		fireUpdateEvent(null);
		
	}
	
	public DataPane() {
		super();
		initialize();
	}
	public DataPane(boolean arg0) {
		super(arg0);
		initialize();
	}
	public DataPane(LayoutManager arg0, boolean arg1) {
		super(arg0, arg1);
		initialize();
	}
	public DataPane(LayoutManager arg0) {
		super(arg0);
		initialize();
	}

	@Override
	public void valueChanged(ListSelectionEvent arg0) {
		int iRow = -1;
		String selId = ""; 
		
		iRow = dataTable.getSelectedRow();
		if(iRow<0){
			//fireUpdateEvent(null);
			return;
		}
		
		try{
			selId = dataTable.getValueAt(iRow, 0).toString();
			int iSel = Integer.parseInt(selId);
			Iu iu = CsMbChecker.worker.getData().getUnits().get(iSel);
			if(iu!=null)fireUpdateEvent(iu);
		}catch (Exception e) {
			//fireUpdateEvent(null);
		}
	}
	
	
}
